#!/usr/bin/perl

#
# copyright (C) vipros@gmail.com @ http://www.vipros.com/photo
# ExifTool is copyrighted under Phil Harvey (http://www.sno.phy.queensu.ca/~phil/exiftool)
#
# 0.1		2005-10-05	Initial Version
#

require 5.004;
require Image::ExifTool;

my $exeDir;
BEGIN {
    # get exe directory
    $exeDir = $0;           # get exe directory from command
    $exeDir =~ tr/\\/\//;   # for systems that use backslashes
    # isolate directory specification
    $exeDir =~ s/(.*)\/.*/$1/ or $exeDir = '.';
    # add lib directory at start of include path
    unshift @INC, "$exeDir/lib";
}

END {
#    Cleanup();
}

use Time::Local;
use Image::ExifTool;


my $gpsDataFile = "./gps.txt";
my $gps2PhotoLog = "./gps2Photo.log.txt";
#my $file = shift or die "Please specify filename";
my $exifTool = new Image::ExifTool;

open FH, $gpsDataFile or die "Can't find file $gpsDataFile: $!\n";

open LOG, ">>$gps2PhotoLog" or die "cannot open $gps2PhotoLog";

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);

if ( $year >= 106 ) {
	print "gps2Photo has expired. Please visit http://www.vipros.com/photo to obtain a new copy. Thank you!";			
	exit;
}

printf LOG "\n\n== Process time: %4d-%02d-%02d %02d:%02d:%02d ==\n", $year+1900,$mon+1,$mday,$hour,$min,$sec;
printf "\n\n== Process time: %4d-%02d-%02d %02d:%02d:%02d ==\n", $year+1900,$mon+1,$mday,$hour,$min,$sec;


# Loop current dir, looking for images
opendir MYDIR, ".";
while($file = readdir(MYDIR)) {
	seek(FH,0,0) or die "Failed to rewind to the beginning of the file";
	next if ($file !~ /^[^GPS_]*\.[j|J][p|P][g|G]$/);

	my $info = $exifTool->ImageInfo($file);
	my $photoTime = $info->{"DateTimeOriginal"};

	my $logString = "\n$file ($photoTime) ";
	printf LOG $logString;
	printf $logString;
	
	my $year;
	my $month;
	my $day;
	my $hour;
	my $minute;
	my $second;
	
	($year,$month,$day, $hour,$minute, $second)=($photoTime =~ /^\s*(\d+):(\d+):(\d+)\s+(\d+):(\d+):(\d+)\s*/x);
	
	$ptime = timegm($second, $minute, $hour, $day, $month-1, $year-1900);

	$found = 0;
	while (<FH>) {
		$gpsstr = $_;
		($loc1, $loc2, $loc3, $loc4, $loc5, $loc6, $loc7, $loc8, $gmonth, $gday, $gyear, $ghour, $gminute, $gsecond, $gapm)=($gpsstr =~ /^Trackpoint.*([N|s])([0-9]+) ([0-9]+)\.([0-9]+) ([W|E])([0-9]+) ([0-9]+)\.([0-9]+).*([0-9]+)\/([0-9]+)\/(200[0-9]) ([0-9]+):([0-9]+):([0-9]+) ([A|P])m*/);
		
		if ( $gyear != "" && $gyear < 2006 ) {
			if ( $ghour < 12 && $gapm == "P" ) {
				$ghour = $ghour + 12;
			}
			$gtime = timegm($gsecond, $gminute, $ghour, $gday, $gmonth - 1, $gyear - 1900);
			$delta = abs($ptime - $gtime);
			for ($i = 1; $i < 60; $i = $i + 5) {
				if ( $delta < $i ) {
					# -GPSLatitudeRef=N -GPSLatitude="43 deg 44.817" -GPSLongitudeRef=W -GPSLongitude="79 deg 22.242"  1.jpg
					$exifTool->SetNewValue("GPSLatitudeRef", "$loc1");
					$exifTool->SetNewValue("GPSLatitude", "$loc2 deg $loc3.$loc4");
					$exifTool->SetNewValue("GPSLongitudeRef", "$loc5");
					$exifTool->SetNewValue("GPSLongitude", "$loc6 deg $loc7.$loc8");
					$exifTool->WriteInfo($file, "GPS_$file");
					$found = "true";
					
					$logString = "taken at $loc1$loc2 $loc3.$loc4 $loc5$loc6 $loc7.$loc8 ($gyear\-$gmonth\-$gday $ghour:$gminute:$gsecond) ($delta sec)";
					#print "$file ($delta sec): photoTime=$photoTime gpsTime=$gyear\-$gmonth\-$gday $ghour:$gminute:$gsecond\n";
					printf LOG $logString;
					printf $logString;
					last;
				}
			}
			if ( $found ) {
				last;
			}
		} # if gyear == 2005
	}

} # end of loop dir
closedir MYDIR;




